@@ -513,6 +513,11 @@ class Blockchain(object):
         
         """
         
+        # This holds all the callbacks we need to call, so we can call them
+        # while we're not holding the lock. It's a list of function, argument
+        # tuple tuples.
+        to_call = []
+        
         with self.lock:
             
             # This is the stack of hashes that we have added to the blockchain.
@@ -526,7 +531,9 @@ class Blockchain(object):
                     self.add_block(next_block)
                     
                     if callback is not None:
-                        callback(next_block.block_hash(), True)
+                        # Later, call the callback.
+                        to_call.append((callback, (next_block.block_hash(),
+                            True)))
                     
                     print "Block height {} immediately verified: {}".format(
                         next_block.height,
@@ -537,7 +544,9 @@ class Blockchain(object):
                 else:
                     print "Invalid block:\n{}".format(next_block)
                     if callback is not None:
-                        callback(next_block.block_hash(), False)
+                        # Later, call the callback.
+                        to_call.append((callback, (next_block.block_hash(),
+                            False)))
             else:
                 # Add this block to the pending blocks
                 self.pending_on[next_block.previous_hash].append(next_block)
@@ -582,8 +591,9 @@ class Blockchain(object):
                             self.add_block(waiter)
                             
                             if waiter_callback is not None:
-                                # Call the callback
-                                waiter_callback(waiter.block_hash(), True)
+                                # Call the callback later
+                                to_call.append((waiter_callback, 
+                                    (waiter.block_hash(), True)))
                             
                             print "Queued block height {} verified: {}".format(
                                 waiter.height,
@@ -601,12 +611,20 @@ class Blockchain(object):
                                 pybc.util.bytes2string(waiter.block_hash()))
                             
                             if waiter_callback is not None:
-                                # Call the callback
-                                waiter_callback(waiter.block_hash(), False)
+                                # Call the callback later
+                                to_call.append((waiter_callback, 
+                                    (waiter.block_hash(), False)))
                     else:
                         # This should never happen
                         print("Couldn't verify a waiting block when its parent "
                             "came in!")
+                            
+        # Now we're out of the locked section.
+        print "Dispatching {} block validity callbacks.".format(len(to_call))
+        for callback, args in to_call:
+            # Call all the callbacks, in case they need to get a lock that
+            # another thread has and that thread is waiting on this thread.
+            callback(*args)
     
     def state_after(self, block):
         """
@@ -845,7 +863,6 @@ class Blockchain(object):
                     
                     # Note: Expect some console spam when debugging invalid
                     # transactions, which are only really expected here.
-                    print "Dropping invalid queued transactions."
                     invalid_transaction_hashes = [hash for hash, transaction in 
                         self.transactions.iteritems() if not 
                         self.verify_transaction(transaction, 
@@ -853,7 +870,10 @@ class Blockchain(object):
                         
                     # Then, remove them all
                     for transaction_hash in invalid_transaction_hashes:
-                        del self.transactions[transaction_hash]   
+                        del self.transactions[transaction_hash]
+                        
+                    print "Dropped {} invalid queued transactions".format(len(
+                        invalid_transaction_hashes))
                 
             else:
                 # Block we tried to add failed verification. Complain.
@@ -1000,11 +1020,13 @@ class Blockchain(object):
                 self.transactions[transaction_hash] = \
                     transaction
                 
-                # Notify the callback
-                if callback is not None:
-                    callback(transaction_hash, True)
+                # Record that it was verified
+                verified = True
+            
             else:
-                # The transaction is invalid.
-                if callback is not None:
-                    # Tell the callback
-                    callback(transaction_hash, False)
+                # Record that it wasn't verified.
+                verified = False
+                
+        # Notify the callback outside the critical section.
+        if callback is not None:
+            callback(transaction_hash, verified)
