@@ -311,11 +311,19 @@ class Peer(object):
                 for connection in self.connections:
                     connection.send_message(message) 
             
-            # Sync the blockchain to disk.
+            
+            print "Saving..."
+            
+            # Sync the blockchain to disk. This needs to lock the blockchain, so
+            # the blockchain must never wait on the peer while holding its lock.
+            # Hence the complicated deferred callback system. TODO: replace the
+            # whole business with events.
             self.blockchain.sync()
             
             # Sync the known peers to disk
             self.known_peers.sync()
+            
+            print "Saved to disk"
                         
             # Tick again later
             reactor.callLater(self.tick_period, self.tick)
